/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: EjbBC.java ofabre$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.ejb;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.jbi.JBIException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.spi.InitialContextFactory;

import org.ow2.petals.bc.ejb.su.EjbServiceUnitListener;
import org.ow2.petals.component.framework.bc.AbstractBindingComponent;
import org.ow2.petals.component.framework.exception.TechnicalException;
import org.ow2.petals.component.framework.su.BindingComponentServiceUnitManager;
import org.ow2.petals.component.framework.util.Extensions;
import org.ow2.petals.jbi.descriptor.Provides;

import com.thoughtworks.xstream.XStream;

/**
 * An EJB binding component
 * 
 * @author ofabre - eBM Websourcing
 * 
 */
public class EjbBC extends AbstractBindingComponent {

    private static final String EJB_JNDI_NAME_PROPERTY = "ejb.jndi.name";

    private Map<Provides, Object> ejbPool;

    private Map<Provides, XStream> xstreamPool;

    private Map<String, ClassLoader> suClassLoaders;

    @Override
    protected void doInit() throws JBIException {
        ejbPool = new HashMap<Provides, Object>();
        suClassLoaders = new HashMap<String, ClassLoader>();
        xstreamPool = new HashMap<Provides, XStream>();
        setServiceUnitListener(new EjbServiceUnitListener(this));
    }

    public Object getEjb(Provides provides) {
        return ejbPool.get(provides);
    }

    public void addEjb(Provides provides, Object ejb) {
        ejbPool.put(provides, ejb);
    }

    public void removeEjb(Provides provides) {
        ejbPool.remove(provides);
    }

    public void removeClassLoader(String suName) {
        suClassLoaders.remove(suName);
    }

    public ClassLoader getSuClassLoader(Provides provides) {
        String suName = ((BindingComponentServiceUnitManager) getServiceUnitManager())
                .getSuDataHandlerForConsumeProvide(provides).getName();
        return suClassLoaders.get(suName);
    }

    public void addSuClassLoader(String suName, ClassLoader classLoader) {
        suClassLoaders.put(suName, classLoader);
    }

    public XStream getXstream(Provides provides) {
        return xstreamPool.get(provides);
    }

    public void addXstream(Provides provides, XStream xStream) {
        xstreamPool.put(provides, xStream);
    }

    public void removeXstream(Provides provides) {
        xstreamPool.remove(provides);
    }
    
    public Context createInitialContext(Extensions extensions) throws EjbBCException {
        Properties environment = new Properties();
        environment.put(InitialContext.PROVIDER_URL, extensions.getValue(Context.PROVIDER_URL));
        environment.put(InitialContext.INITIAL_CONTEXT_FACTORY, extensions
                .getValue(InitialContext.INITIAL_CONTEXT_FACTORY));
        return createJNDIContext(environment);
    }

    /**
     * Return a JNDI context loaded from the JAR files contained in the
     * ServiceUnit and the Properties defined in the JBI.XML file.
     * 
     * @param properties
     *            contains the factory class name and the url provider
     * @return a JNDI context
     * @throws TechnicalException
     *             can not retrieved the factory. properties might not be
     *             correct.
     * 
     */
    public Context createJNDIContext(Properties properties) throws EjbBCException {
        Context ctx = null;
        try {
            ClassLoader cl = this.getClass().getClassLoader();
            Class clazz = cl.loadClass(properties
                    .getProperty(InitialContext.INITIAL_CONTEXT_FACTORY));
            InitialContextFactory icf = (InitialContextFactory) clazz.newInstance();
            ctx = icf.getInitialContext(properties);
        } catch (Exception e) {
            throw new EjbBCException("JNDI context can not be created." + e, e);
        }
        return ctx;
    }

    public Object retrieveObject(Context initialContext, Extensions extensions)
            throws EjbBCException {
        try {
            return initialContext.lookup(extensions.getValue(EJB_JNDI_NAME_PROPERTY));
        } catch (NamingException e) {
            throw new EjbBCException("Can't retrieve ejb for name :"
                    + extensions.getValue(EJB_JNDI_NAME_PROPERTY), e);
        }
    }
}
